import random, math, sys, optparse

import pygame
from   pygame.locals import *

from   OpenGL.GL import *
from   OpenGL.GLU import *

import config

import ResourceManager, Geometry, Scene

def gl_setup(fullscreen = True):
   if fullscreen:
      disp = pygame.display.set_mode(config.VIDEO_MODE, OPENGL | DOUBLEBUF | FULLSCREEN)
   else:
      disp = pygame.display.set_mode(config.VIDEO_MODE, OPENGL | DOUBLEBUF)
      
   glEnable(GL_CULL_FACE)
   
   glEnable(GL_DEPTH_TEST)
   glDepthFunc(GL_LEQUAL)
   
   glEnable(GL_BLEND)
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

   #glEnable(GL_TEXTURE_2D)
   
   glEnable(GL_DEPTH_TEST)
   glDepthFunc(GL_LEQUAL)
   
   glShadeModel(GL_SMOOTH)

   glEnable(GL_LINE_SMOOTH)
   glLineWidth(1.5)
      
   #glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
   #glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
   #glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
   #glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)

   glEnable(GL_POINT_SMOOTH)
   glPointSize(5.0)

   #glClipPlane(GL_CLIP_PLANE0, [0.0, 1.0, 0.0, -BOARD_POS[1] + BOARD_VGAP * 0.4])

   #glClearColor(0, 0, 0, 0)
   glClearColor(0.0, 0.2, 0, 0.1)

   glMatrixMode(GL_PROJECTION)
   glLoadIdentity()

   ratio = float(config.VIDEO_MODE[0]) / config.VIDEO_MODE[1]

   glOrtho(config.SCREEN_LEFT, config.SCREEN_RIGHT, -0.5, 0.5, -100.0, 100.0)
   glMatrixMode(GL_MODELVIEW)

   glLoadIdentity()
   
   #glLightfv(GL_LIGHT0, GL_AMBIENT, (0.2, 0.2, 0.2, 1.0))
   glLightfv(GL_LIGHT0, GL_AMBIENT, (0.0, 0.0, 0.0, 0.0))
   glLightfv(GL_LIGHT0, GL_DIFFUSE, (1.0, 1.0, 1.0, 1.0))
   glLightfv(GL_LIGHT0, GL_POSITION, (0.0, 0.0, 1.0, 0.0))
  
   glEnable(GL_LIGHT0)
   glEnable(GL_LIGHTING)
      
   glMaterialfv(GL_FRONT, GL_AMBIENT, (0.8, 0.8, 0.8, 1.0))

   return disp

def main():
   parser = optparse.OptionParser()
 
   parser.add_option("-f", "--fullscreen",
      action = "store_true",
      dest = "fullscreen",
      help = "Start the game in fullscreen mode. Default is windowed mode."
   )

   parser.add_option("--640",
      action = "store_const",
      dest = "resolution",
      const = (640, 480),
      help = "640x480 video mode. (Default)"
   )
   
   parser.add_option("--800",
      action = "store_const",
      dest = "resolution",
      const = (800, 600),
      help = "800x600 video mode."
   )
   
   parser.add_option("--1024",
      action = "store_const",
      dest = "resolution",
      const = (1024, 768),
      help = "1024x768 video mode."
   )

   parser.set_defaults(
      resolution = (640, 480),
      fullscreen = False,
      music = True
   )

   options, args = parser.parse_args()

   config.enable_music = options.music
   
   pygame.init()
   try:
      pygame.mixer.set_num_channels(16)
   except pygame.error, e:
      print "Warning: %s" % e
  
   config.setVideoMode(options.resolution)
   display = gl_setup(options.fullscreen)
   pygame.display.set_caption(config.WINDOW_TITLE)

   ResourceManager.SoundManager()
   ResourceManager.FontManager()
   ResourceManager.SourceManager()

   gm = ResourceManager.GeometryManager()
   gm.add("tetra", Geometry.Tetrahedron())
   gm.add("cube", Geometry.Cube())
   gm.add("octa", Geometry.Octahedron())
   gm.add("ring", Geometry.Ring(0.5, 24))

   scene = Scene.MenuScene()
   while scene is not None:
      scene = scene.run()

